(*
 * <COPYRIGHT>
 * Copyright 2005 Gerd Stolpmann
 *
 * <GPL>
 * This file is part of UMLMON.
 *
 * UMLMON is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * UMLMON is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with UMLMON; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * </>
 *)

(* $Id: umlmon_client.ml 191 2005-10-14 19:04:52Z gerd $ *)

open Umlmonctrl_aux
open Cryptokit

module Cl = Umlmonctrl_clnt.Ctrl.V1
module Dir = Umlmonctrl_clnt.Directory.V1

let at_id_re = Str.regexp "^\\([^@]+\\)@\\(.*\\)$"

let vardir() =
  try
    Sys.getenv "UMLMON_DIR"
  with
      Not_found -> "/var/lib/umlmon"
;;

let create_client ?(prefer_local_socket = false) 
                    ?(prefer_redirection = false)
                     ues id =
  let (vmid, host, uds) =
    if Str.string_match at_id_re id 0 then (
      let vmid = Str.matched_group 1 id in
      let host = Str.matched_group 2 id in
      let uds = host = "localhost" && prefer_local_socket in
      (vmid, host, uds)
    )
    else
      (id, "localhost", true) in

  if uds then (    
    (* Unix Domain *)
    Cl.create_client 
      ~esys:ues
      (Rpc_client.Unix (Filename.concat (vardir()) (vmid ^ "/ctrl"))) Rpc.Tcp
  )
  else (
    let serv = 
      try Unix.getservbyname "umldir" "tcp"
      with Not_found -> failwith "umldir/tcp service not found (probably in /etc/services)" in
    let dir = Dir.create_client 
		~esys:ues
		(Rpc_client.Inet(host, serv.Unix.s_port))
		Rpc.Tcp in
    let have_redir = Dir.have_redirection dir () in
    if have_redir then (
      Rpc_client.shut_down dir;
      let serv_redir = 
	try Unix.getservbyname "umlredir" "tcp"
	with Not_found -> failwith "umlredir/tcp service not found (probably in /etc/services)" in
      let client = 
	Cl.create_client 
	  ~esys:ues
	  (Rpc_client.Inet(host,serv_redir.Unix.s_port))
	  Rpc.Tcp in
      ( try
	  Cl.ensure_id client vmid
	with
	  | Rpc_client.Message_lost
	  | Rpc_client.Client_is_down ->
	      Rpc_client.shut_down client;
	      failwith "VM instance not found in directory"
      );
      client
    )
    else (
      let port_opt = Dir.get_port dir vmid in
      Rpc_client.shut_down dir;
      match port_opt with
	| None ->
	    failwith "VM instance not found in directory"
	| Some port ->
	    Cl.create_client 
	      ~esys:ues
	      (Rpc_client.Inet(host,Int32.to_int port))
	      Rpc.Tcp
    )
  )
;;


let need_authentication cl =
  try
    ignore(Cl.get_id cl ());
    false
  with
      Rpc.Rpc_server Rpc.Auth_failed ->
	true
;;


let authenticate ?(userpw=false) client passwords =
  let challenge = Cl.auth_get_challenge client () in
  let responses =
    List.map
      (fun pw ->
	 let pw_md5 = Digest.string pw in
	 let mac = MAC.hmac_sha1 challenge in
	 let prefix = if userpw then "userpw:" else "" in
	 prefix ^ hash_string mac pw_md5
      )
      passwords in
  Cl.auth_pass_credentials client ("HMAC-SHA1", Array.of_list responses)
;;


let authenticate'async ?(userpw=false) client passwords action =
  Cl.auth_get_challenge'async client () 
    (fun f ->
       try
	 let challenge = f() in
	 let responses =
	   List.map
	     (fun pw ->
		let pw_md5 = Digest.string pw in
		let mac = MAC.hmac_sha1 challenge in
		let prefix = if userpw then "userpw:" else "" in
		prefix ^ hash_string mac pw_md5
	     )
	     passwords in
	 Cl.auth_pass_credentials'async
	   client ("HMAC-SHA1", Array.of_list responses) action
       with
	   err -> action (fun () -> raise err)
    )
;;


let change_password client old_password new_password =
  let old_pw_md5 = Digest.string old_password in
  let new_pw_md5 = Digest.string new_password in
  let trans = Cipher.aes ~mode:Cipher.CBC old_pw_md5 Cipher.Encrypt in
  let new_pw_enc = transform_string trans new_pw_md5 in
  let r = Cl.auth_change_password client new_pw_enc in
  match r with
    | `code_ok -> ()
    | `code_error msg -> failwith ("change_password: " ^ msg)
    | _ -> failwith "change_password: unexpected reply"
;;
